-------------------------------------------------------------------------------
-- Title   : Top-up monitoring
-- Project : TimEX3
-------------------------------------------------------------------------------
-- Description : Monitoring of top-up
--               Pre top-up  => p_State signal is set with inTTL(0)
--               Post top-up => p_State signal is reset with inTTL(1)
--               topUpError(0) set = time between pre top-up and post top-up exceed maximum
--               topUpError(1) set = time between post top-up and pre top-up exceed maximum
-------------------------------------------------------------------------------
-- File     : topup_monitiring.vhdl
-- Revision : x.x.x
-- Created  : October 26, 2012
-- Updated  : March 11, 2013
-------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
-------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
-------------------------------------------------------------------------------
--
-------------------------------------------------------------------------------
-- Modifications :
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.registers_init.all;

-------------------------------------------------------------------------------
--------------------------------- ENTITY --------------------------------------
-------------------------------------------------------------------------------
entity topup_monitor is
  port (
    p_clk1kHz           : in  std_logic;
    p_clk60MHz          : in  std_logic;
    p_inTTL             : in  std_logic_vector (1  downto 0);
    p_maxPostTopUpDelay : in  std_logic_vector (31 downto 0); -- delay between post and pre top-up signals
    p_maxPreTopUpDelay  : in  std_logic_vector (31 downto 0); -- delay between pre and post top-up signals
    p_csw               : in  std_logic_vector (1  downto 0);
    p_reset             : in  std_logic;
    p_State             : out std_logic; -- 0 : post top-up ; 1 : pre top-up
    p_topUpError        : out std_logic_vector (1 downto 0) -- Error(0) : pre to post error ;
                                                            -- Error(1) : post to pre error
  );
end entity topup_monitor ;

-------------------------------------------------------------------------------
------------------------------- ARCHITECTURE ----------------------------------
-------------------------------------------------------------------------------
architecture rtl_topup_monitor of topup_monitor is
  -----------------------------------------------------------------------------
  -- constant
  -----------------------------------------------------------------------------

  -----------------------------------------------------------------------------
  -- signal
  -----------------------------------------------------------------------------
  signal s_set             : std_logic;
  signal s_cntPostTopUp    : unsigned (31 downto 0);
  signal s_cntPostTopUpMax : unsigned (31 downto 0);
  signal s_cntPreTopUp     : unsigned (31 downto 0);
  signal s_cntPreTopUpMax  : unsigned (31 downto 0);
  signal s_errorPostTopUp  : std_logic;
  signal s_errorPreTopUp   : std_logic;
  signal s_rstCNT0         : std_logic;
  signal s_rstCNT1         : std_logic;

  -----------------------------------------------------------------------------
  -- Components
  -----------------------------------------------------------------------------

  -----------------------------------------------------------------------------
  --------------------------------- Main --------------------------------------
  -----------------------------------------------------------------------------
  begin

   -- Set process
   -- p_inTTL(0) = pre top-up ; p_inTTL(1) = post top-up
  process (p_reset, p_inTTL(1), p_inTTL(0))
  begin
    if ((p_reset = '1') or (p_inTTL(1) = '1')) then
      s_set <= '0'; -- post top-up
    elsif (rising_edge(p_inTTL(0))) then
      s_set <= '1'; -- pre top-up
    else
      s_set <= s_set;
    end if;
  end process;

  -- Time out
  process (p_reset, s_rstCNT0, s_rstCNT1, p_clk1kHz)
  begin
    if ((p_reset = '1') or (s_rstCNT0 = '1') or (s_rstCNT1 = '1')) then
      s_cntPreTopUp  <= X"00000001";
      s_cntPostTopUp <= X"00000001";
    elsif (rising_edge(p_clk1kHz)) then
      if ((s_set = '0') and (s_errorPostTopUp = '0')) then
        -- Counting the post top-up time
        s_cntPostTopUp <= s_cntPostTopUp + 1;
        s_cntPreTopUp  <= X"00000001";
      end if;
      if ((s_set = '1') and (s_errorPreTopUp = '0')) then
        --  Counting the pre top-up time
        s_cntPreTopUp  <= s_cntPreTopUp + 1;
        s_cntPostTopUp <= X"00000001";
      end if;
    end if;
  end process;

  -- Error detection
  process (p_reset, s_rstCNT0, s_rstCNT1, p_clk1kHz)
  begin
   if ((p_reset = '1') or (s_rstCNT0 = '1') or (s_rstCNT1 = '1')) then
     s_errorPostTopUp <= '0';
     s_errorPreTopUp  <= '0';
   elsif (rising_edge(p_clk1kHz)) then
      if (s_cntPostTopUp > s_cntPostTopUpMax) then
        -- Post top-up error (time between post top-up and pre top-up)
        s_errorPostTopUp <= '1';
      else
        s_errorPostTopUp <= '0';
      end if;
      if (s_cntPreTopUp > s_cntPreTopUpMax) then
        -- Pre top-up error (time between pre top-up and post top-up)
        s_errorPreTopUp <= '1';
      else
        s_errorPreTopUp <= '0';
      end if;
    end if;
  end process;

  -- Latch from the register the maximum delay before a missing post top-up
  -- signal is signaled (waiting for pre top-up sigbal ; delay between post
  -- and pre top-up signals)
  process (p_reset, p_clk60MHz)
  begin
    if (p_reset = '1') then
      s_cntPostTopUpMax <= unsigned (c_topUpMaxPostTopUpDelay_init);
      s_rstCNT0         <= '0';
    elsif (rising_edge(p_clk60MHz)) then
      if (p_csw(0) = '1') then
        s_cntPostTopUpMax <= unsigned (p_maxPostTopUpDelay);
        s_rstCNT0         <= '1';
      else
        s_cntPostTopUpMax <= s_cntPostTopUpMax;
        s_rstCNT0         <= '0';
      end if;
    end if;
  end process;

  -- Latch from the register the maximum delay before a missing pre top-up
  -- signal is signaled (waiting for the post top-up ; delay between pre and
  -- post top-up signals)
  process (p_reset, p_clk60MHz)
  begin
    if (p_reset = '1') then
      s_cntPreTopUpMax <=  unsigned (c_topUpMaxPreTopUpDelay_init);
      s_rstCNT1        <= '0';
    elsif (rising_edge(p_clk60MHz)) then
      if (p_csw(1) = '1') then
        s_cntPreTopUpMax <= unsigned (p_maxPreTopUpDelay);
        s_rstCNT1        <= '1';
      else
        s_cntPreTopUpMax <= s_cntPreTopUpMax;
        s_rstCNT1        <= '0';
      end if;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------
  p_topUpError <= s_errorPostTopUp & s_errorPreTopUp;
  p_State      <= s_set; -- 0 : post top-up ; 1 : pre top-up

end architecture rtl_topup_monitor;
