--------------------------------------------------------------------------------
-- Title   : Top-up gating top file
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : Top-up gating top sheet block
--               output(2) : positive gating window
--               output(3) : negative gating window
--               output(4) : machine gating window
--------------------------------------------------------------------------------
-- File     : top_topup.vhdl
-- Revision : x.x.x
-- Created  : October 26, 2012
-- Updated  : March 11, 2013
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity topup_top is
  port (
    p_topUp_inTTL             : in  std_logic_vector (1 downto 0);
    p_topUp_outTTL            : out std_logic_vector (4 downto 0);
    p_topUp_outPECL           : out std_logic_vector (4 downto 0);
    p_topUp_clk1kHz           : in  std_logic;
    p_topUp_clk500mHz         : in  std_logic;
    p_topUp_clk60MHz          : in  std_logic;
    p_topUp_maxPostTopUpDelay : in  std_logic_vector (31 downto 0);
    p_topUp_maxPreTopUpDelay  : in  std_logic_vector (31 downto 0);
    p_topUp_gateHoldOff       : in  std_logic_vector (31 downto 0);
    p_topUp_gateWidth         : in  std_logic_vector (31 downto 0);
    p_topUp_csw               : in  std_logic_vector (3  downto 0);
    p_topUp_reset             : in  std_logic;
    p_topup_Error             : out std_logic_vector (1 downto 0);
    -- LED(0) = red ; LED(1) = green
    p_topUp_LED               : out std_logic_vector (1 downto 0)
   );
end entity topup_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_topup_top of topup_top is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_State       : std_logic;
  signal s_in1         : std_logic;
  signal s_in0         : std_logic;
  signal s_topup_Error : std_logic_vector (1 downto 0);
  signal s_gateWindow  : std_logic;

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  attribute KEEP : string;
  attribute KEEP of s_in0, s_in1 : signal is "TRUE";

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

    -- Gating of the top-up injection
    topup_gating : entity work.topup_gating (rtl_topup_gating)
      port map (
        p_clk1kHz     => p_topUp_clk1kHz,
        p_clk60MHz    => p_topUp_clk60MHz,
        p_inTTL(0)    => p_topUp_inTTL(0),
        p_inTTL(1)    => s_in1,
        p_gateWindow  => s_gateWindow,
        p_gateHoldOff => p_topUp_gateHoldOff,
        p_gateWidth   => p_topUp_gateWidth,
        p_csw         => p_topUp_csw(3 downto 2),
        p_reset       => p_topUp_reset
      );

    -- Motiring of the top-up injection
    -- Rise error if pre top-up or post top-up delays are above user specified one
    topup_monitor : entity work.topup_monitor (rtl_topup_monitor)
      port map (
        p_clk1kHz           => p_topUp_clk1kHz,
        p_clk60MHz          => p_topUp_clk60MHz,
        p_inTTL(0)          => p_topUp_inTTL(0),
        p_inTTL(1)          => s_in1,
        p_maxPostTopUpDelay => p_topUp_maxPostTopUpDelay, -- delay between post and pre top-up signals
        p_maxPreTopUpDelay  => p_topUp_maxPreTopUpDelay,  -- delay between pre and post top-up signals
        p_csw               => p_topUp_csw(1 downto 0),
        p_reset             => p_topUp_reset,
        p_State             => s_State,      -- 0 : post top-up ; 1 : pre top-up
        p_topUpError        => s_topup_Error -- Error(0) : pre to post error ;
                                             -- Error(1) : post to pre error
      );

    -- Turn on the LEDs according to the top-up monitiring
    topup_leds : entity work.topup_leds (rtl_topup_leds)
      port map (
        p_clk500mHz => p_topUp_clk500mHz,
        p_State     => s_State,
        p_Error     => s_topup_Error,
        p_ledOut    => p_topUp_LED
      );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  s_in0 <= not p_topup_inTTL(1);
  s_in1 <= not s_in0;

  p_topUp_outTTL  (1 downto 0) <= "00"; -- not used
  p_topUp_outTTL  (4 downto 2) <= s_State & (not s_gateWindow) & s_gateWindow;
  p_topUp_outPECL (4 downto 0) <= "00000";

  p_topup_Error <= s_topup_Error;

end architecture rtl_topup_top;
